/*****
*
* Copyright (C) 1998 - 2000 Vandoorselaere Yoann <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <errno.h>

#include "report.h"

#define plural(number) (number > 1) ? "s" : ""

static char *program = NULL;


/*
 *
 */
static void execmod_run(alert_t *alert, report_infos_t *rinfos) {
        int i;
        char msg[1024];
        plugin_generic_t *p = alert_plugin(alert);

        
        i = snprintf(msg, sizeof(msg), "Date\t: %s\n", rinfos->date_start);
        if ( rinfos->date_end )
                i += snprintf(msg + i, sizeof(msg) - i, " - %s\n", rinfos->date_end);
        else
                i += snprintf(msg + i, sizeof(msg) - i, "\n");
         
        snprintf(msg + i, sizeof(msg) - i, "Plugin\t: %s\n", plugin_name(p));
        snprintf(msg + i, sizeof(msg) - i, "Author\t: %s\n", plugin_author(p));
        snprintf(msg + i, sizeof(msg) - i, "Contact\t: %s\n", plugin_contact(p));
        snprintf(msg + i, sizeof(msg) - i, "description\t: %s\n", plugin_desc(p));
        snprintf(msg + i, sizeof(msg) - i, "kind\t\t: %s\n", rinfos->kind);
        snprintf(msg + i, sizeof(msg) - i, "received\t: %d time%s\n", alert_count(alert), plural(alert_count(alert)));
        snprintf(msg + i, sizeof(msg) - i, "message\t\t: %s\n\n", alert_message(alert));

        i = execlp(program, msg, NULL);
        if ( i < 0 )
                log(LOG_ERR, "couldn't exec %s.\n", program);
}




static void print_help(const char *pname) 
{
        fprintf(stderr, "Usage for %s :\n", pname);
        fprintf(stderr, "\t -e --enable Enable this plugin.\n");
        fprintf(stderr, "\t -p --program Program to execute.\n");
}



static int check_opts(char *pname) 
{
        char **argv;
        config_t *cfg;        
        int c, argc, enable = 0;
        struct option opts[] = {
                { "enable", no_argument, NULL, 'e' },
                { "program", required_argument, NULL, 'p' },
                { "help", no_argument, NULL, 'h' },
                { 0, 0, 0, 0 },
        };

        plugin_get_opts(pname, &argc, &argv);
        
        while ( (c = getopt_long(argc, argv, "p:eh", opts, NULL)) != -1 ) {
                switch (c) {
                case 'e':
                        enable = 1;
                        break;

                case 'p':
                        program = optarg;
                        break;
                        
                case 'h':
                        print_help(pname);
                        break;
                        
                default:
                        return -1;
                }
        }

        if ( (! enable || ! program ) && (cfg = config_open(PRELUDE_REPORT_CONF)) ) {
                const char *str;

                
                if ( ! enable && (str = config_get(cfg, pname, "enable") )) {
                        if ( strcmp(str, "yes") == 0 )
                                enable = 1;
                } else {
                        config_close(cfg);
                        return -1;
                }
                
                if ( ! program && (str = config_get(cfg, pname, "program") ))
                        program = strdup(str);

                config_close(cfg);
        }
        
        return (enable == 1 && program) ? 0 : -1;
}



int plugin_init(unsigned int id)
{
        int ret;
        static plugin_report_t plugin;
    
        plugin_set_name(&plugin, "ExecMod");
        
        ret = check_opts(plugin.name);
        if ( ret < 0 )
                return -1;
        
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "Will log all alert to a file.");
        plugin_set_running_func(&plugin, execmod_run);
        plugin_set_closing_func(&plugin, NULL);
        
	plugin_register((plugin_generic_t *)&plugin);

        return 0;
}












